From 95dd4e043ea4635c44b656d0145c6bb67408e47f Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Mattias=20Engdeg=C3=A5rd?= <mattiase@acm.org>
Date: Mon, 23 Sep 2019 18:56:30 +0200
Subject: [PATCH] Allow regexp-quote to return its argument

* src/search.c (Fregexp_quote): Only allocate a new string if needed.
---
 doc/lispref/searching.texi |  3 +++
 etc/NEWS                   |  4 ++++
 src/search.c               | 10 ++++++----
 3 files changed, 13 insertions(+), 4 deletions(-)

diff --git a/doc/lispref/searching.texi b/doc/lispref/searching.texi
index 015871fab0..a4b6533412 100644
--- a/doc/lispref/searching.texi
+++ b/doc/lispref/searching.texi
@@ -1717,6 +1717,9 @@ Regexp Functions
  (concat "\\s-" (regexp-quote string) "\\s-"))
 @end group
 @end example
+
+The returned string may be @var{string} itself if it does not contain
+any special characters.
 @end defun
 
 @cindex optimize regexp
diff --git a/etc/NEWS b/etc/NEWS
index 3f7d4894df..49d733b8ac 100644
--- a/etc/NEWS
+++ b/etc/NEWS
@@ -2156,6 +2156,10 @@ valid event type.
 ---
 ** The obsolete package xesam.el (since Emacs 24) has been removed.
 
++++
+** 'regexp-quote' may return its argument string.
+If the argument needs no quoting, it can be returned instead of a copy.
+
 
 * Lisp Changes in Emacs 27.1
 
diff --git a/src/search.c b/src/search.c
index 1e57d2ecbe..9d95dcbca5 100644
--- a/src/search.c
+++ b/src/search.c
@@ -3138,10 +3138,12 @@ DEFUN ("regexp-quote", Fregexp_quote, Sregexp_quote, 1, 1, 0,
     }
 
   Lisp_Object result
-    = make_specified_string (temp,
-			     SCHARS (string) + backslashes_added,
-			     out - temp,
-			     STRING_MULTIBYTE (string));
+    = (backslashes_added > 0
+       ? make_specified_string (temp,
+                                SCHARS (string) + backslashes_added,
+                                out - temp,
+                                STRING_MULTIBYTE (string))
+       : string);
   SAFE_FREE ();
   return result;
 }
-- 
2.21.0 (Apple Git-122)

